---------------------------------------------------------------
-- NON RESTORING Division algorithm for natural (unsigned) X and Y
-- NBITS >= MBITS
-- Returns a Quotient Q of NBITS and a Remiander R de MBITS
--	The correction step is only an adition
---------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.mypackage.all;

entity div_nr_unsigned is
    port (
        X: in STD_LOGIC_VECTOR (NBITS-1 downto 0);
        Y: in STD_LOGIC_VECTOR (MBITS-1 downto 0);
        Q: out STD_LOGIC_VECTOR (NBITS-1 downto 0);
		  R: out STD_LOGIC_VECTOR (MBITS-1 downto 0)
     );
end div_nr_unsigned;

architecture mult_arch of div_nr_unsigned is

COMPONENT cond_adder
	PORT(
		a : IN std_logic_vector(MBITS-1 downto 0);
		b : IN std_logic_vector(MBITS-1 downto 0);
		x_i : IN std_logic;          
		r : OUT std_logic_vector(MBITS-1 downto 0)
		);
END COMPONENT;

component nr_cell is
    port (
        a: in STD_LOGIC_VECTOR (MBITS downto 0);
        b: in STD_LOGIC_VECTOR (MBITS-1 downto 0);
        q: in STD_LOGIC;
		  r: out STD_LOGIC_VECTOR (MBITS downto 0)
     );
end component;

type conections is array (0 to NBITS-1) of STD_LOGIC_VECTOR (MBITS downto 0);
Signal wires_in, wires_out: conections;

Signal  QQ: STD_LOGIC_VECTOR (NBITS downto 0);
signal  adjust: STD_LOGIC;
Signal  zeros: STD_LOGIC_VECTOR (MBITS-1 downto 0);

begin

 QQ(NBITS) <= '0';	 
 zeros <= (others => '0'); 
 wires_in(0) <= zeros & X(NBITS-1);  

 divisor: for I in 0 to NBITS-1 generate
    int_mod: nr_cell port map (a => wires_in(I),
  			 b => Y, 
  			 q => QQ(NBITS-I),
  			 r => wires_out(i) );
  end generate;
  			 

 wires_conections: for I in 0 to NBITS-2 generate
    QQ(NBITS-i-1) <= wires_out(i)(MBITS);	 
    wires_in(i+1) <= wires_out(i)(MBITS-1 downto 0) & X(NBITS-I-2);	 
  end generate;

 adjust <= (wires_out(NBITS-1)(MBITS));
  
 final_adjust: cond_adder port map ( a => wires_out(NBITS-1)(MBITS-1 downto 0),
  			 b => Y, x_i => adjust, r => R);

 Q(NBITS-1 downto 1) <= not QQ(NBITS-1 downto 1);
 Q(0) <= not adjust;

end mult_arch;


