-----------------------------------------------------------------------------
-- VHDL Test Bench NON-Restoring divisor for  signed X and unsigned Y
--
-- Exhaustive Analysis
-- Notes: 		
--	if NBITS and/or MBITS are big, It will need a los of disk space and time
--
-----------------------------------------------------------------------------
LIBRARY  IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE IEEE.std_logic_signed.all;
USE work.my_package.all;

LIBRARY ieee;
USE IEEE.STD_LOGIC_TEXTIO.ALL;
USE STD.TEXTIO.ALL;

ENTITY testbench_nr_integer IS
END testbench_nr_integer;

ARCHITECTURE behavior OF testbench_nr_integer IS 
FILE RESULTS: TEXT OPEN WRITE_MODE IS "results.txt";

	COMPONENT div_nr_integer
	PORT(
        X: in STD_LOGIC_VECTOR (NBITS-1 downto 0);
        Y: in STD_LOGIC_VECTOR (MBITS-1 downto 0);
        Q: out STD_LOGIC_VECTOR (NBITS-1 downto 0);
        R: out STD_LOGIC_VECTOR (MBITS-1 downto 0)
		);
	END COMPONENT;

	CONSTANT DELAY: time := 100 ns;
	CONSTANT print_ok: boolean := false;
   SIGNAL A: STD_LOGIC_VECTOR (NBITS-1 downto 0);
   SIGNAL B: STD_LOGIC_VECTOR (MBITS-1 downto 0);
   SIGNAL Q: STD_LOGIC_VECTOR (NBITS-1 downto 0);
   SIGNAL R: STD_LOGIC_VECTOR (MBITS-1 downto 0);

BEGIN

   uut: div_nr_integer PORT MAP( X => A, Y => B, Q => Q, R => R );


   tb_test : PROCESS
	VARIABLE TX_LOC : LINE;
	VARIABLE TX_STR : String(1 to 4096);
	Variable iQ, iR: integer;
   BEGIN
		WAIT FOR 10 ns;
		for I in -(2**(NBITS-1))+1 to 2**(NBITS-1)-1 loop
			for J in -(2**(MBITS-1))+1 to 2**(MBITS-1)-1 loop
				A <= CONV_STD_LOGIC_VECTOR (I, NBITS);
				B <= CONV_STD_LOGIC_VECTOR (J, MBITS);
				WAIT FOR DELAY/2;
			   if J /= 0 then
					iQ := IEEE.std_logic_signed.CONV_INTEGER(Q);
					iR := IEEE.std_logic_signed.CONV_INTEGER(R);
					if (i /= (J * iQ) + iR) THEN 
						write(TX_LOC,string'("Error!!! -> A=")); write(TX_LOC, A);
						write(TX_LOC,string'(" B=")); write(TX_LOC, B);
						write(TX_LOC,string'(" Q=")); write(TX_LOC, Q);
						write(TX_LOC,string'(" R=")); write(TX_LOC, R);
						write(TX_LOC,string'(" (i=")); write(TX_LOC, i);
						write(TX_LOC,string'(" j=")); write(TX_LOC, j);
						write(TX_LOC,string'(" iQ=")); write(TX_LOC, iQ);
						write(TX_LOC,string'(" iR=")); write(TX_LOC, iR);
						write(TX_LOC,string'(" iQ * J=")); write(TX_LOC, J * iQ);
						write(TX_LOC, string'(") "));
						TX_STR(TX_LOC.all'range) := TX_LOC.all;
						writeline(results, TX_LOC);
						Deallocate(TX_LOC);
						ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
					elsif (abs(iR) > abs(J)) then
						write(TX_LOC,string'("Error Remainder bigger than Divider !!! -> A=")); write(TX_LOC, A);
						write(TX_LOC,string'(" B=")); write(TX_LOC, B);
						write(TX_LOC,string'(" Q=")); write(TX_LOC, Q);
						write(TX_LOC,string'(" R=")); write(TX_LOC, R);
						write(TX_LOC,string'(" (i=")); write(TX_LOC, i);
						write(TX_LOC,string'(" j=")); write(TX_LOC, j);
						write(TX_LOC,string'(" iQ=")); write(TX_LOC, iQ);
						write(TX_LOC,string'(" iR=")); write(TX_LOC, iR);
						write(TX_LOC, string'(") "));
						TX_STR(TX_LOC.all'range) := TX_LOC.all;
						writeline(results, TX_LOC);
						Deallocate(TX_LOC);
						ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;

				   elsif (print_ok) then
						write(TX_LOC,string'("OK -> A=")); write(TX_LOC, A);
						write(TX_LOC,string'(" B=")); write(TX_LOC, B);
						write(TX_LOC,string'(" Q=")); write(TX_LOC, Q);
						write(TX_LOC,string'(" R=")); write(TX_LOC, R);
						write(TX_LOC, string'(" "));
						TX_STR(TX_LOC.all'range) := TX_LOC.all;
						writeline(results, TX_LOC);
						Deallocate(TX_LOC);
						ASSERT (FALSE) REPORT TX_STR SEVERITY WARNING;
					END IF;
				END IF;
				
				WAIT FOR DELAY/2;

    		end loop;
   	end loop;
		ASSERT (FALSE) REPORT
		"Simulation successful (not a failure).  No problems detected. "
		SEVERITY FAILURE;
   END PROCESS;

END;